@@ -132,6 +132,8 @@ pub fn message_impl(args: TokenStream, input: TokenStream) -> TokenStream {
     let crate_ = args.crate_;
     let internal = quote![#crate_::_priv];
 
+    let protocol = std::env::var("CARGO_PKG_NAME").expect("building without cargo?");
+
     let impl_request = if let Some(ret) = &args.ret {
         assert!(!args.part, "`part` and `ret` attributes are incompatible");
 
@@ -211,16 +213,26 @@ pub fn message_impl(args: TokenStream, input: TokenStream) -> TokenStream {
         quote! {
             impl #crate_::Message for #name {
                 const _LTID: #internal::LocalTypeId = #ltid;
+                const PROTOCOL: &'static str = #protocol;
+                const NAME: &'static str = stringify!(#name);
+                const LABELS: &'static [#internal::metrics::Label] = &[
+                    #internal::metrics::Label::from_static_parts("message", Self::NAME),
+                    #internal::metrics::Label::from_static_parts("protocol", Self::PROTOCOL),
+                ];
             }
 
             #[doc(hidden)]
             #[allow(non_snake_case)]
             mod #mod_name {
                 use super::*;
+                use #crate_::Message;
 
                 use std::fmt;
 
-                use #internal::{MESSAGE_LIST, MessageVTable, smallbox::{smallbox}, AnyMessage, linkme};
+                use #internal::{
+                    MESSAGE_LIST, MessageVTable, AnyMessage, dumping,
+                    smallbox::smallbox, linkme
+                };
 
                 #request_wrapper
 
@@ -236,15 +248,23 @@ pub fn message_impl(args: TokenStream, input: TokenStream) -> TokenStream {
                     fmt::Debug::fmt(cast_ref(message), f)
                 }
 
+                fn erase(message: &AnyMessage) -> dumping::ErasedMessage {
+                    smallbox!(Clone::clone(cast_ref(message)))
+                }
+
                 #[linkme::distributed_slice(MESSAGE_LIST)]
                 #[linkme(crate = #internal::linkme)]
                 static VTABLE: MessageVTable = MessageVTable {
-                    ltid: #ltid,
+                    ltid: #name::_LTID,
+                    name: #name::NAME,
+                    protocol: #name::PROTOCOL,
+                    labels: #name::LABELS,
                     clone,
                     debug,
+                    erase,
                 };
 
-                // See [https://github.com/rust-lang/rust/issues/47384](rust#47384).
+                // See [rust#47384](https://github.com/rust-lang/rust/issues/47384).
                 #[doc(hidden)]
                 pub fn touch() {}
                 impl #name { #[doc(hidden)] pub fn _elfo_touch() { #mod_name::touch(); } }
